@@ -47,7 +47,7 @@ STATUS_TO_BUTTON = dict((s, ANSWER_TO_BUTTON[a])
 
 # HACK: Setting and unsetting previous and next buttons to workaround
 #       for gnome bug #56070.
-class GTKHack(object):
+class GTKHack:
     def __init__(self, function):
         self._function = function
 
@@ -211,6 +211,19 @@ class GTKInterface(UserInterface):
             title += " - %s" % test_name
         self._get_widget("dialog_main").set_title(title)
 
+    def _set_progress(self,progress):
+        # Update progress bar
+        bar = self._get_widget("progressbar_test")
+        if not self.progress: 
+            return
+        done, total = self.progress
+        bar.set_text("%(done)d/%(total)d" % {'done': done, 'total': total})
+        if total:
+            progress_fraction = float(done) / total
+        else:
+            progress_fraction=0
+        bar.set_fraction(progress_fraction)
+
     def _run_dialog(self, dialog=None):
         def on_dialog_response(dialog, response, self):
             # Keep dialog alive when the button that has been clicked
@@ -221,8 +234,16 @@ class GTKInterface(UserInterface):
                 self.direction = response
                 Gtk.main_quit()
 
+        def on_dialog_key_press(dialog, event, self):
+            # Ignore ESC key presses
+            key = Gdk.keyval_name(event.keyval)
+            if "Escape" in key:
+                return True
+            return False
+
         dialog = dialog or self._dialog
         dialog.connect("response", on_dialog_response, self)
+        dialog.connect("key-press-event", on_dialog_key_press, self)
         dialog.set_default_response(NEXT)
         dialog.show()
         Gtk.main()
@@ -230,6 +251,7 @@ class GTKInterface(UserInterface):
             raise KeyboardInterrupt
 
     def show_progress_start(self, message):
+        self._set_progress(self.progress)
         self._set_sensitive("button_previous", False)
         self._set_sensitive("button_next", False)
 
@@ -266,7 +288,7 @@ class GTKInterface(UserInterface):
         self._set_button("button_next", next_state)
 
     @GTKHack
-    def show_entry(self, text, value, previous=None, next=None):
+    def show_entry(self, text, value, label=None, previous=None, next=None):
         #Reset window title
         self._set_main_title()
         # Set buttons
@@ -315,7 +337,7 @@ class GTKInterface(UserInterface):
 
         # Set callbacks
         def click_button(widget, active):
-            for check_button in option_table.itervalues():
+            for check_button in option_table.values():
                 check_button.set_active(active)
 
         for button_name in "button_select_all", "button_deselect_all":
@@ -375,11 +397,11 @@ class GTKInterface(UserInterface):
         return result
 
     @GTKHack
-    def show_tree(self, text, options={}, default={}):
+    def show_tree(self, text, options={}, default={}, deselect_warning=""):
         #Reset window title
         self._set_main_title()
 
-        (COLUMN_TEXT, COLUMN_ACTIVE) = range(2)
+        (COLUMN_TEXT, COLUMN_ACTIVE) = list(range(2))
 
         # Set buttons
         self._notebook.set_current_page(1)
@@ -519,6 +541,7 @@ class GTKInterface(UserInterface):
             self._set_hyper_text_view("hyper_text_view_test",
                 test["description"])
 
+        self._set_progress(self.progress) 
         # Set buttons
         if "command" in test:
             self._set_sensitive("button_test", True)
@@ -566,15 +589,35 @@ class GTKInterface(UserInterface):
         self.direction = NEXT
         return options[response]
 
-    def show_error(self, text):
+    def show_error(self, primary_text,
+                   secondary_text=None, detailed_text=None):
         message_dialog = Gtk.MessageDialog(parent=self._dialog,
             type=Gtk.MessageType.ERROR,
             buttons=Gtk.ButtonsType.NONE,
-            message_format=text)
+            message_format=primary_text)
         message_dialog.set_modal(True)
         message_dialog.set_title(_("Error"))
         message_dialog.set_default_response(NEXT)
         message_dialog.add_buttons(Gtk.STOCK_CLOSE, NEXT)
+        if secondary_text:
+            message_dialog.format_secondary_text(secondary_text)
+        if detailed_text:
+            content_area = message_dialog.get_content_area()
+
+            expander = Gtk.Expander(label=_('Detailed information...'))
+
+            def expanded_cb(expander, *args):
+                message_dialog.set_resizable(expander.get_expanded())
+            expander.connect("notify::expanded", expanded_cb)
+            scrolled_window = Gtk.ScrolledWindow()
+            textview = Gtk.TextView()
+            textview.set_editable(False)
+            textview.get_buffer().set_text(detailed_text)
+            scrolled_window.add(textview)
+            expander.add(scrolled_window)
+            content_area.pack_start(expander,
+                                    expand=True, fill=True, padding=0)
+        message_dialog.show_all()
         self._run_dialog(message_dialog)
         message_dialog.hide()
 
