@@ -27,7 +27,27 @@ from checkbox.user_interface import (UserInterface, NEXT, PREV,
                                      ALL_ANSWERS, ANSWER_TO_STATUS)
 
 
-class Dialog(object):
+# HACK to workaround bug in urwid.container, line 1273
+class ComparableString(str):
+
+    def __init__(self, string):
+        self.string = string
+
+    def __getattr__(self, name):
+        return getattr(self.string, name)
+
+    def __lt__(self, other):
+        if not isinstance(other, str):
+            return False
+        return self.string < other
+
+    def __gt__(self, other):
+        if not isinstance(other, str):
+            return False
+        return self.string > other
+
+
+class Dialog:
     """
     Basic dialog class that displays some text
     """
@@ -77,8 +97,10 @@ class Dialog(object):
         if not body:
             text = urwid.Text(self.text)
             walker = urwid.SimpleListWalker([])
+            list_box = urwid.ListBox(walker)
+            list_box.pref_col = ComparableString(list_box.pref_col)
             body = urwid.Pile((('flow', text),
-                               ('weight', 1, urwid.ListBox(walker))))
+                               ('weight', 1, list_box)))
             self.walker = walker
 
         frame = urwid.AttrMap(urwid.Frame(body, header, footer), 'body')
@@ -290,9 +312,9 @@ class TestDialog(ChoiceDialog):
         """
         Return the label of the selected radio button
         """
-        label = (radio_button.get_label()
+        label = next((radio_button.get_label()
                  for radio_button in self.radio_button_group
-                 if radio_button.get_state()).next()
+                 if radio_button.get_state()))
         return label
 
 
@@ -379,7 +401,7 @@ class TreeChoiceDialog(ChoiceDialog):
                                      widget.changed_cb, self.walker)
 
         if isinstance(data, dict):
-            items = sorted(data.iteritems(), key=itemgetter(0))
+            items = sorted(iter(data.items()), key=itemgetter(0))
             for children_name, children_data in items:
                 child_widget = self.create_tree(children_name, children_data, widget)
                 widget.append(child_widget)
@@ -392,7 +414,7 @@ class TreeChoiceDialog(ChoiceDialog):
         Set selected nodes by default recursively
         """
         if isinstance(default, dict):
-            for name, default_children in default.iteritems():
+            for name, default_children in default.items():
                 for widget in widgets:
                     if widget.name == name:
                         widget.state = True
@@ -408,7 +430,7 @@ class TreeChoiceDialog(ChoiceDialog):
 
         # Show tree
         self.option_widgets = []
-        items = sorted(self.options.iteritems(),
+        items = sorted(iter(self.options.items()),
                        key=itemgetter(0))
         for name, data in items:
             widget = self.create_tree(name, data)
@@ -498,10 +520,10 @@ class ReportDialog(ChoiceDialog):
         urwid.signals.connect_signal(widget, 'change',
                                      widget.changed_cb, self.walker)
 
-        items = sorted(data.iteritems(), key=itemgetter(0))
+        items = sorted(iter(data.items()), key=itemgetter(0))
         for child_name, child_data in items:
             is_suite = all(issubclass(type(value), dict)
-                           for value in child_data.itervalues())
+                           for value in child_data.values())
 
             if is_suite:
                 child_widget = self.create_tree(child_name,
@@ -533,7 +555,7 @@ class ReportDialog(ChoiceDialog):
         Dialog.show(self)
 
         # Show tree
-        items = sorted(self.results.iteritems(),
+        items = sorted(iter(self.results.items()),
                        key=itemgetter(0))
         for name, data in items:
             widget = self.create_tree(name, data)
@@ -899,7 +921,7 @@ class ProgressDialog(Dialog):
         """
         Pulse progress bar
         """
-        start, end = self.progress_coordinates.next()
+        start, end = next(self.progress_coordinates)
         bar_data = [[0]]*start + [[1]]*(end-start) + [[0]]*(self.MAX_VALUE-end)
         self.progress_bar.set_data(bar_data, 1)
         self.loop.draw_screen()
@@ -955,7 +977,7 @@ class UrwidInterface(UserInterface):
         return dialog.response
 
 
-    def show_entry(self, text, value, previous=None, next=None):
+    def show_entry(self, text, value, label=None, previous=None, next=None):
         dialog = InputDialog(text).run()
         self.direction = dialog.direction
         return dialog.response
@@ -974,7 +996,7 @@ class UrwidInterface(UserInterface):
         return dialog.response
 
 
-    def show_tree(self, text, options={}, default={}):
+    def show_tree(self, text, options={}, default={}, deselect_warning=""):
         """
         Show some options in a tree hierarchy
         and let the user choose between them
@@ -1004,8 +1026,7 @@ class UrwidInterface(UserInterface):
             output = ''
 
         # Get options
-        options = list([self.ANSWER_TO_OPTION[a]
-                        for a in ALL_ANSWERS])
+        options = [self.ANSWER_TO_OPTION[a] for a in ALL_ANSWERS]
 
         # Get buttons
         buttons = []
@@ -1040,9 +1061,11 @@ class UrwidInterface(UserInterface):
         """
         return self.show_radio(text, options, default)
 
-
-    def show_error(self, text):
+    def show_error(self, primary_text,
+                   secondary_text=None, detailed_text=None):
         """
         Show an error message
         """
+        text = filter(None, [primary_text, secondary_text, detailed_text])
+        text = '\n'.join(text)
         return self.show_radio(text)
