@@ -30,10 +30,8 @@ from argparse import ArgumentParser
 from argparse import FileType
 from argparse import _ as argparse_gettext
 from fnmatch import fnmatch
-from io import TextIOWrapper
 from logging import basicConfig
 from logging import getLogger
-from os import listdir
 from os.path import join
 import argparse
 import sys
@@ -43,9 +41,7 @@ from plainbox.impl.checkbox import CheckBox
 from plainbox.impl.commands import PlainBoxCommand
 from plainbox.impl.commands.selftest import SelfTestCommand
 from plainbox.impl.exporter import get_all_exporters
-from plainbox.impl.job import JobDefinition
 from plainbox.impl.result import JobResult
-from plainbox.impl.rfc822 import load_rfc822_records
 from plainbox.impl.runner import JobRunner
 from plainbox.impl.runner import slugify
 from plainbox.impl.session import SessionState
@@ -73,13 +69,8 @@ class CheckBoxCommandMixIn:
         """
         group = parser.add_argument_group(title="job definition options")
         group.add_argument(
-            "--load-extra", action="append",
-            metavar="FILE", default=[],
-            help="Load extra job definitions from FILE",
-            type=FileType("rt"))
-        group.add_argument(
-            '-r', '--run-pattern', action="append",
-            metavar='PATTERN', default=[], dest='run_pattern_list',
+            '-i', '--include-pattern', action="append",
+            metavar='PATTERN', default=[], dest='include_pattern_list',
             help="Run jobs matching the given pattern")
         # TODO: Find a way to handle the encoding of the file
         group.add_argument(
@@ -89,50 +80,24 @@ class CheckBoxCommandMixIn:
             help="Load whitelist containing run patterns")
 
     def get_job_list(self, ns):
-        # Load built-in job definitions
-        job_list = self.get_builtin_jobs()
-        # Load additional job definitions
-        job_list.extend(self._load_jobs(ns.load_extra))
-        return job_list
-
-    def get_builtin_jobs(self):
-        logger.debug("Loading built-in jobs...")
-        return self._load_builtin_jobs()
-
-    def _load_jobs(self, source_list):
         """
-        Load jobs from the list of sources
+        Load and return a list of JobDefinition instances
         """
-        job_list = []
-        for source in source_list:
-            job_list.extend(self.load(source))
-        return job_list
-
-    def _load_builtin_jobs(self):
-        """
-        Load jobs from built into CheckBox
-        """
-        return self._load_jobs([
-            join(self.checkbox.jobs_dir, name)
-            for name in listdir(self.checkbox.jobs_dir)
-            if name.endswith(".txt") or name.endswith(".txt.in")])
+        return self._checkbox.get_builtin_jobs()
 
     def _get_matching_job_list(self, ns, job_list):
         # Find jobs that matched patterns
         matching_job_list = []
         if ns.whitelist:
-            ns.run_pattern_list.extend([pattern.strip() for pattern in
+            ns.include_pattern_list.extend([pattern.strip() for pattern in
                                         ns.whitelist.readlines()])
         for job in job_list:
-            for pattern in ns.run_pattern_list:
+            for pattern in ns.include_pattern_list:
                 if fnmatch(job.name, pattern):
                     matching_job_list.append(job)
                     break
         return matching_job_list
 
-    def load(self, something):
-        return load(something)
-
 
 class SpecialCommand(PlainBoxCommand, CheckBoxCommandMixIn):
 
@@ -180,7 +145,7 @@ class SpecialCommand(PlainBoxCommand, CheckBoxCommandMixIn):
         # specified just operate on the whole set. The ns.special check
         # prevents people starting plainbox from accidentally running _all_
         # jobs without prompting.
-        if ns.special is not None and not ns.run_pattern_list:
+        if ns.special is not None and not ns.include_pattern_list:
             matching_job_list = job_list
         return matching_job_list
 
@@ -310,7 +275,7 @@ class RunCommand(PlainBoxCommand, CheckBoxCommandMixIn):
                 outcome_callback = self.ask_for_outcome
             else:
                 outcome_callback = None
-            runner = JobRunner(self.checkbox, session.session_dir,
+            runner = JobRunner(session.session_dir,
                                session.jobs_io_log_dir,
                                outcome_callback=outcome_callback)
             self._run_jobs_with_session(ns, session, runner)
@@ -379,16 +344,20 @@ class RunCommand(PlainBoxCommand, CheckBoxCommandMixIn):
 
     def _run_single_job_with_session(self, ns, session, runner, job):
         print("[ {} ]".format(job.name).center(80, '-'))
+        if job.description is not None:
+            print(job.description)
+            print("^" * len(job.description.splitlines()[-1]))
+            print()
         job_state = session.job_state_map[job.name]
-        print("Job name: {}".format(job.name))
-        print("Plugin: {}".format(job.plugin))
-        print("Direct dependencies: {}".format(job.get_direct_dependencies()))
-        print("Resource dependencies: {}".format(
-            job.get_resource_dependencies()))
-        print("Resource program: {!r}".format(job.requires))
-        print("Command: {!r}".format(job.command))
-        print("Can start: {}".format(job_state.can_start()))
-        print("Readiness: {}".format(job_state.get_readiness_description()))
+        logger.debug("Job name: %s", job.name)
+        logger.debug("Plugin: %s", job.plugin)
+        logger.debug("Direct dependencies: %s", job.get_direct_dependencies())
+        logger.debug("Resource dependencies: %s",
+                     job.get_resource_dependencies())
+        logger.debug("Resource program: %r", job.requires)
+        logger.debug("Command: %r", job.command)
+        logger.debug("Can start: %s", job_state.can_start())
+        logger.debug("Readiness: %s", job_state.get_readiness_description())
         if job_state.can_start():
             if ns.dry_run:
                 print("Not really running anything in dry-run mode")
@@ -444,7 +413,8 @@ class PlainBox:
         # To compensate, on python3.3 and beyond, when the user just runs
         # plainbox without specifying the command, we manually, explicitly do
         # what python3.2 did: call parser.error(_('too few arguments'))
-        if sys.version_info[:2] >= (3, 3) and getattr(ns, "command", None) is None:
+        if (sys.version_info[:2] >= (3, 3)
+                and getattr(ns, "command", None) is None):
             parser.error(argparse_gettext("too few arguments"))
         else:
             return ns.command.invoked(ns)
@@ -488,27 +458,5 @@ def save(something, somewhere):
     raise NotImplementedError("save() not implemented")
 
 
-def load(somewhere):
-    if isinstance(somewhere, str):
-        # Load data from a file with the given name
-        filename = somewhere
-        with open(filename, 'rt', encoding='UTF-8') as stream:
-            return load(stream)
-    if isinstance(somewhere, TextIOWrapper):
-        stream = somewhere
-        logger.debug("Loading jobs definitions from %r...", stream.name)
-        record_list = load_rfc822_records(stream)
-        job_list = []
-        for record in record_list:
-            job = JobDefinition.from_rfc822_record(record)
-            logger.debug("Loaded %r", job)
-            job_list.append(job)
-        return job_list
-    else:
-        raise TypeError(
-            "Unsupported type of 'somewhere': {!r}".format(
-                type(somewhere)))
-
-
 def run(*args, **kwargs):
     raise NotImplementedError("run() not implemented")
