@@ -16,12 +16,20 @@
 # You should have received a copy of the GNU General Public License
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
 #
+import logging
+
 from checkbox.contrib.persist import Persist, MemoryBackend
 
 from checkbox.job import JobStore, PASS, UNINITIATED, UNTESTED
 from checkbox.properties import Int, Path
 from checkbox.plugin import Plugin
-from checkbox.user_interface import NEXT, PREV
+from checkbox.user_interface import (
+    NEXT,
+    PREV,
+    CONTINUE_ANSWER,
+    RESTART_ANSWER,
+    RERUN_ANSWER,
+)
 
 
 class JobsPrompt(Plugin):
@@ -43,7 +51,7 @@ class JobsPrompt(Plugin):
     def store(self):
         if self._store is None:
             self._store = JobStore(self.persist, self.store_directory,
-                self.store_directory_size)
+                                   self.store_directory_size)
 
         return self._store
 
@@ -53,16 +61,18 @@ class JobsPrompt(Plugin):
         self._ignore = []
         self._persist = None
         self._store = None
+        self._fail_current = False
 
-        for (rt, rh) in [
-             ("begin-persist", self.begin_persist),
-             ("begin-recover", self.begin_recover),
-             ("ignore-jobs", self.ignore_jobs),
-             ("prompt-job", self.prompt_job),
-             ("prompt-jobs", self.prompt_jobs),
-             ("prompt-finish", self.prompt_finish),
-             ("report", self.report),
-             ("report-job", self.report_job)]:
+        for (rt, rh) in [("expose-msgstore", self.expose_msgstore),
+                         ("begin-persist", self.begin_persist),
+                         ("begin-recover", self.begin_recover),
+                         ("ignore-jobs", self.ignore_jobs),
+                         ("prompt-job", self.prompt_job),
+                         ("prompt-jobs", self.prompt_jobs),
+                         ("prompt-finish", self.prompt_finish),
+                         ("report", self.report),
+                         ("report-job", self.report_job),
+                         ("report-jobs", self.report_jobs)]:
             self._manager.reactor.call_on(rt, rh)
 
         #This should fire first thing during the gathering phase.
@@ -72,11 +82,19 @@ class JobsPrompt(Plugin):
         #all other gathering callbacks are finished)
         self._manager.reactor.call_on("gather", self.end_gather, 900)
 
+    def expose_msgstore(self):
+        self._manager.reactor.fire("store-access", self.store)
+
     def begin_persist(self, persist):
         self._persist = persist
 
     def begin_recover(self, recover):
-        if not recover:
+        if recover == RERUN_ANSWER:
+            logging.debug("Recovering from last job")
+        elif recover == CONTINUE_ANSWER:
+            logging.debug("Marking last job failed, starting from next job")
+            self._fail_current = True
+        else:
             self.store.delete_all_messages()
 
     def begin_gather(self):
@@ -85,7 +103,7 @@ class JobsPrompt(Plugin):
 
     def end_gather(self):
         #Back to saving data very carefully once gathering is done.
-        self.store.safe_file_closing = True 
+        self.store.safe_file_closing = True
 
     def ignore_jobs(self, jobs):
         self._ignore = jobs
@@ -95,7 +113,9 @@ class JobsPrompt(Plugin):
         job.setdefault("status", UNINITIATED)
         self._manager.reactor.fire("report-%s" % job["plugin"], job)
 
-        self.store.add(job)
+    def report_jobs(self, jobs):
+        for job in jobs:
+            self.store.add(job)
 
     def prompt_job(self, interface, job):
         attribute = "description" if job.get("type") == "suite" else "name"
@@ -111,11 +131,12 @@ class JobsPrompt(Plugin):
                 # Skip if any message in the depends doesn't pass
                 depends = job["depends"]
                 for message in messages:
-                    if message["name"] in depends \
-                       and message["status"] != PASS:
+                    if message["name"] in depends and \
+                       message["status"] != PASS:
                         return
 
-            self._manager.reactor.fire("prompt-%s" % job["plugin"], interface, job)
+            self._manager.reactor.fire("prompt-%s" % job["plugin"],
+                                       interface, job)
 
     def prompt_jobs(self, interface):
         while True:
@@ -123,10 +144,26 @@ class JobsPrompt(Plugin):
                 if not self.store.remove_pending_offset():
                     break
 
+            if self._fail_current:
+                msg_to_fail = self.store.get_pending_messages(1)
+                job_to_fail = msg_to_fail[0]
+                job_to_fail["status"] = "fail"
+                logging.warning("Marking job %s as failed"
+                                "at user request" % job_to_fail["name"])
+                self.store.update(job_to_fail)
+                self.store.add_pending_offset()
+                self._fail_current = False
+
             messages = self.store.get_pending_messages(1)
             if not messages:
                 break
 
+            done_count = self.store.get_pending_offset()
+            pending_count = self.store.count_pending_messages()
+
+            progress = (done_count, done_count + pending_count)
+            self._manager.reactor.fire("set-progress", progress)
+
             job = messages[0]
             self._manager.reactor.fire("prompt-job", interface, job)
             self.store.update(job)
@@ -149,7 +186,8 @@ class JobsPrompt(Plugin):
         suites = [m for m in messages if m.get("type") == "suite"]
         self._manager.reactor.fire("report-suites", suites)
 
-        attachments = [m for m in messages if m.get("type") == "attachment" and "data" in m]
+        attachments = [m for m in messages
+                      if m.get("type") == "attachment" and "data" in m]
         self._manager.reactor.fire("report-attachments", attachments)
 
 
